import os
import uuid


def new_request_id() -> uuid.UUID:
    """
    Generate a new request ID.

    A request ID is a potentially user-facing ID of request-response-cycle
    initiated by a client. In essence, a request ID is a version 4 UUID with
    the last 48 bits zeroed out. This allows for a neat prefix search on
    sub-request IDs generated by :func:`new_sub_request_id`.

    :return: A new request ID.
    :rtype: uuid.UUID
    """
    return uuid.UUID(bytes=os.urandom(10) + b"\x00" * 6, version=4)


def new_sub_request_id(request_id: uuid.UUID) -> uuid.UUID:
    """
    Generate a new sub-request ID.

    Sub-request IDs are IDs used by services internally and should never leave
    a service's boundary. They are essentially version 4 UUIDs where the first
    160 bits are identical to the related request ID as generated by
    :func:`new_request_id`.

    Generating sub-request IDs with the same prefix as the related request ID
    allows for prefix searches. Multiple sub-request IDs within the same
    request ID have the same prefix.

    :param uuid.UUID request_id: The request ID to be used as basis for the UUID
        generation.
    :return: A new sub-request ID.
    :rtype: uuid.UUID
    """
    return uuid.UUID(bytes=request_id.bytes[:10] + os.urandom(6), version=4)
